(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@svogv/core'), require('@angular/common'), require('@angular/forms')) :
    typeof define === 'function' && define.amd ? define('@svogv/bootstrap', ['exports', '@angular/core', '@svogv/core', '@angular/common', '@angular/forms'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.svogv = global.svogv || {}, global.svogv.bootstrap = {}), global.ng.core, global['@svogv/core'], global.ng.common, global.ng.forms));
}(this, (function (exports, core, core$1, common, forms) { 'use strict';

    /**
     * The pagination component creates a few buttons to navigate a grid. The underlaying model
     * is going to handle the date on the client. The pagination does not support a server backend,
     * all relevant data must be loaded first.
     *
     * Example of usage:
     * @example
     * ```html
     * <ac-pagination></ac-pagination>
     * ```
     *
     * <example-url>/#/widget/grid</example-url>
     */
    var DataGridPaginationComponent = /** @class */ (function () {
        function DataGridPaginationComponent() {
            /**
             * An event fired once the user has changed the page by clicking a button.
             */
            this.pageNumberChanged = new core.EventEmitter();
            this.currentPageNumber = 1;
        }
        DataGridPaginationComponent.prototype.ngOnInit = function () {
            this.setCurrentPage(1);
        };
        DataGridPaginationComponent.prototype.ngOnChanges = function (changes) {
            var _this = this;
            if (changes['maxPageIndex']) {
                var change = changes['maxPageIndex'];
                if (this.currentPageNumber > change.currentValue) {
                    // throws ExpressionChangedAfterItHasBeenCheckedException
                    // if there's no setTimeout.
                    // no need to add setTimeout if ngOnChanges
                    // is fired after changes made on root component.
                    setTimeout(function () { return _this.setCurrentPage(1); }, 1);
                }
            }
        };
        DataGridPaginationComponent.prototype.setCurrentPage = function (pageNumber, event) {
            if (event) {
                event.preventDefault();
            }
            if (pageNumber === 0 || pageNumber > this.maxPageIndex
                || pageNumber === this.currentPageNumber) {
                return;
            }
            this.pageNumberChanged.emit(pageNumber);
            this.currentPageNumber = pageNumber;
        };
        DataGridPaginationComponent.prototype.range = function (min, max) {
            var result = new Array();
            for (var i = min; i <= max; i++) {
                result.push(i);
            }
            return result;
        };
        Object.defineProperty(DataGridPaginationComponent.prototype, "pageStartNumber", {
            get: function () {
                var startNumber = this.currentPageNumber <= 4
                    ? 1
                    : this.currentPageNumber >= this.maxPageIndex - 3
                        ? this.maxPageIndex - 6
                        : this.currentPageNumber - 3;
                return startNumber < 1 ? 1 : startNumber;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(DataGridPaginationComponent.prototype, "pageEndNumber", {
            get: function () {
                var pageEnd = this.pageStartNumber + 6;
                return pageEnd > this.maxPageIndex ? this.maxPageIndex : pageEnd;
            },
            enumerable: false,
            configurable: true
        });
        return DataGridPaginationComponent;
    }());
    DataGridPaginationComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ac-datagrid-pagination',
                    template: "<div>\n  <ul class=\"pagination float-right\" [ngClass]=\"{ 'pagination-sm': size == 'sm', 'pagination-lg': size == 'lg' }\">\n    <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\" class=\"page-item\">\n      <a href (click)=\"setCurrentPage(1, $event)\" aria-label=\"Previous\" class=\"page-link\">\n        <span aria-hidden=\"true\">\u00AB</span>\n      </a>\n    </li>\n    <li [class.disabled]=\"currentPageNumber === 1 || !maxPageIndex\" class=\"page-item\">\n      <a\n        href\n        aria-label=\"Previous\"\n        (click)=\"setCurrentPage(currentPageNumber - 1, $event)\"\n        class=\"page-link\">\n        <span aria-hidden=\"true\">\u2039</span>\n      </a>\n    </li>\n    <li\n      *ngFor=\"let index of range(pageStartNumber, pageEndNumber)\"\n      [class.active]=\"currentPageNumber === index\"\n      class=\"page-item\">\n      <a href (click)=\"setCurrentPage(index, $event)\" class=\"page-link\">\n        <span aria-hidden=\"true\">{{ index }}</span>\n      </a>\n    </li>\n    <li [class.disabled]=\"currentPageNumber === maxPageIndex || !maxPageIndex\">\n      <a class=\"page-link\"\n        href\n        (click)=\"setCurrentPage(currentPageNumber + 1, $event)\"\n        aria-label=\"Last\"\n      >\n        <span aria-hidden=\"true\">\u203A</span>\n      </a>\n    </li>\n    <li [class.disabled]=\"currentPageNumber === maxPageIndex || !maxPageIndex\" class=\"page-item\">\n      <a href (click)=\"setCurrentPage(maxPageIndex, $event)\" aria-label=\"Last\" class=\"page-link\">\n        <span aria-hidden=\"true\">\u00BB</span>\n      </a>\n    </li>\n  </ul>\n</div>\n",
                    styles: [""]
                },] }
    ];
    DataGridPaginationComponent.propDecorators = {
        maxPageIndex: [{ type: core.Input }],
        pageNumberChanged: [{ type: core.Output }],
        size: [{ type: core.Input }]
    };

    /**
     * A classic data grid. You provide a model to handle all features. The model is build from a
     * simple array of objects with decorators.
     *
     * > See 'Documentation and Examples' tab for a complete documentation.
     *
     * ### Summary
     *
     * The datagrid provides basic functions for data tables:
     *
     * * sorting
     * * filtering
     * * pagination
     * * editing
     *
     * Provide an decorator enhanced model and the grid appears driven by model meta data.
     *
     * There are many attributes and ways to change the appearance. Also some classes can be controlled by
     * the host component:
     *
     * * `.col-borders`
     * * `.col-last`
     * * `.col-first`
     *
     * All these styles are applied to the <col> elements of the underlying table.
     *
     * The model used in the example is an array of objects, where the properties are decorated with
     * various decorators used to control the grid's render behavior.
     *
     * ~~~typescript
     * const data: UserViewModel[] = this.dataSource; // provide a simple array here
     *  this.model = new DataGridModel<UserViewModel>(data, UserViewModel);
     * ~~~
     *
     * The class {@link DataGridModel} controls the grid. You must provide a viewmodel, this is mandatory. The viewmodel is being examined at runtime,
     * so assure you provide a class and set all properties to a default to force creation of properties.
     *
     * ~~~typescript
     * export class UserViewModel {
     *  email: string = ''; // the = '' is necessary!
     *  // more omitted for brevity
     * }
     * ~~~
     *
     * The grid can be extended with the {@link DataGridPaginationComponent} to page through huge data sets. The model
     * handles the pagination, the additional {@link DataGridPaginationComponent} is only a predefined renderer that
     * supports the used theme.
     *
     * <example-url>/#/widget/grid</example-url>
     *
     * @example
     *  <ac-datagrid
     *      [model]="model"
     *      [showActions]="false"
     *      [columnStyle]=""
     *  ></ac-datagrid>
     */
    var DataGridComponent = /** @class */ (function () {
        function DataGridComponent() {
            /**
             * @ignore
             */
            this.directionEnumHelper = core$1.Direction;
            this.externals = {};
            /**
             * Show the action column at all. Use {@link showDeleteButton and  @see showEditButton to switch the}buttons
             * on or off individually. Default is `true` (actions visible).
             */
            this.showActions = true;
            /**
             * The text that appears on the Delete button. Default is 'Delete'.
             */
            this.textDeleteButton = 'Delete';
            /**
             * The text that appears on the Edit button. Default is 'Edit'.
             */
            this.textEditButton = 'Edit';
            /**
             * The column header of the column that shows the buttons. Default is 'Actions'.
             */
            this.textButtonsHeader = 'Actions';
            /**
             * The text that appears if there are no items to show. Can also be overwritten by a more complex piece
             * of code by adding a template like this:
             *
             * @example
             * <ng-template #data-warning-noitems>
             *   <div class="alert alert-danger">The grid is empty</div>
             * </ng-template>
             */
            this.textNoItems = 'There are no items to show';
            /**
             * Event forwarded from model class and being fired after the model class's onEdit event.
             * The event is invoked by the appropriate internal button via click.
             */
            this.editItem = new core.EventEmitter();
            /**
             * Event forwarded from model class and being fired after the model class's onDelete event.
             * The event is invoked by the appropriate internal button via click.
             */
            this.deleteItem = new core.EventEmitter();
            // tslint:disable-next-line:member-ordering
            /**
             * @ignore
             */
            this.warnProp = {};
        }
        Object.defineProperty(DataGridComponent.prototype, "filter", {
            /**
             * The filter value to filter the content. The data is of type `{ [prop: string]: any }`.
             *
             * @param value A dictionary with filter instructions as shown below. The filter logic applies one after another, like an __AND__ conjunction.
             *
             * @example
             * {
             *    "email": "paul@sample.com"
             *    "name": "Paul"
             * }
             *
             */
            set: function (value) {
                if (this.model) {
                    this.model.searchValue = value;
                }
            },
            enumerable: false,
            configurable: true
        });
        DataGridComponent.prototype.ngAfterViewInit = function () {
            var _this = this;
            if (this.model) {
                this.model.onEdit.subscribe(function (item) { return _this.editItem.emit(item); });
                this.model.onDelete.subscribe(function (item) { return _this.deleteItem.emit(item); });
            }
        };
        DataGridComponent.prototype.ngOnDestroy = function () {
            if (this.model) {
                this.model.onEdit.unsubscribe();
                this.model.onDelete.unsubscribe();
            }
        };
        /**
         * @ignore
         * Controls the template used to display certain data types.
         * If the host provides a template it's being used, otherwise a fallback is provided
         * @param uiHint Property of {@link UiHint} decorator
         */
        DataGridComponent.prototype.getActiveTemplate = function (uiHint, prop) {
            if (this[uiHint]) {
                // if provided by user via ContentChild and overwriting defaults (string == string etc.)
                return this[uiHint];
            }
            if (this.externals[uiHint]) {
                // if provided by user via ContentChild but completely replaced
                return this.externals[uiHint];
            }
            if (this[uiHint + "Fallback"]) {
                // otherwise we take ours from ng-template via ViewChild
                return this[uiHint + "Fallback"];
            }
            // if we go here the model requested a custom template that didn't exists
            if (this.warnProp[prop] !== uiHint) {
                console.warn("Property " + prop + " requested the template " + uiHint + ", but it wasn't provided. Falling back to \"string\".");
                this.warnProp[prop] = uiHint;
            }
            return this.stringFallback;
        };
        return DataGridComponent;
    }());
    DataGridComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ac-datagrid',
                    template: "<div class=\"row\" *ngIf=\"model && model.headers\">\n  <div class=\"col-md-12\">\n    <table class=\"table table-condensed\">\n      <colgroup class=\"colborders\" [ngStyle]=\"columnStyle ? columnStyle.group : null\">\n        <col class=\"first\" [ngStyle]=\"columnStyle ? columnStyle.first : null\" />\n        <col class=\"middle\" [ngStyle]=\"columnStyle ? columnStyle.middle : null\" />\n        <col class=\"middle\" [ngStyle]=\"columnStyle ? columnStyle.middle : null\" />\n        <col class=\"middle\" [ngStyle]=\"columnStyle ? columnStyle.middle : null\" />\n        <col class=\"last\" [ngStyle]=\"columnStyle ? columnStyle.last : null\" />\n      </colgroup>\n      <thead>\n        <tr [ngStyle]=\"columnStyle ? columnStyle.header : null\" >\n          <th *ngFor=\"let header of model.headers\"\n              [ngClass]=\"{ 'sortableHeader': header.isSortable }\"\n              [ngStyle]=\"header.uiHint || {}\"\n              [title]=\"header.desc\"\n              (click)=\"header.isSortable ? model.sortColumn(header.prop, null, header.sortCallback) : null\">\n            <span class=\"headerText\">{{ header.text }}</span>\n            <span class=\"sort\" [ngStyle]=\"columnStyle ? columnStyle.headerButton : null\" *ngIf=\"header.isSortable\">\n              <button (click)=\"model.sortColumn(header.prop, directionEnumHelper.Ascending, header.sortCallback)\"\n                      class=\"up\"\n                      [hidden]=\"model.sortDirection[header.prop] === directionEnumHelper.Descending\"\n                      [ngStyle]=\"columnStyle ? columnStyle.headerSortButton : null\">\n                <i class=\"fa fa-sort-asc\"></i>\n              </button>\n              <button (click)=\"model.sortColumn(header.prop, directionEnumHelper.Descending, header.sortCallback)\"\n                      class=\"down\"\n                      [hidden]=\"model.sortDirection[header.prop] === directionEnumHelper.Ascending\"\n                      [ngStyle]=\"columnStyle ? columnStyle.headerSortButton : null\">\n                <i class=\"fa fa-sort-desc\"></i>\n              </button>\n            </span>\n          </th>\n          <th *ngIf=\"showActions\">\n            {{ textButtonsHeader }}\n          </th>\n        </tr>\n      </thead>\n      <tbody>\n        <tr *ngIf=\"!model.totalFilteredRows\">\n          <td [attr.colspan]=\"model.headers.length\" class=\"text-center\">\n            <div class=\"alert alert-warning\">\n              <span #warningNoItems>\n                <ng-content select=\"[data-warning-noitems]\"></ng-content>\n              </span>\n              <ng-container *ngIf=\"!warningNoItems.innerHTML.trim()\">{{  textNoItems }}</ng-container>\n            </div>\n          </td>\n        </tr>\n        <tr *ngFor=\"let item of model.itemsOnCurrentPage\">\n          <td *ngFor=\"let header of model.headers\">\n            <ng-container\n              [ngTemplateOutlet]=\"getActiveTemplate(header.templateHint, header.prop)\"\n              [ngTemplateOutletContext]=\"{ $implicit: item[header.prop], pipe: header.pipe, params: header.pipeParams }\"\n            ></ng-container>\n          </td>\n          <td *ngIf=\"showActions\" [ngStyle]=\"columnStyle ? columnStyle.actionCell : null\" >\n            <button class=\"btn btn-primary btn-sm\" type=\"button\" (click)=\"model.editItem(item)\">\n              {{ textEditButton }}\n            </button>\n            <button class=\"btn btn-danger btn-sm\" type=\"button\" (click)=\"model.deleteItem(item)\">\n              {{ textDeleteButton }}\n            </button>\n          </td>\n        </tr>\n      </tbody>\n      <tfoot>\n        <tr [ngStyle]=\"columnStyle ? columnStyle.footer : null\" >\n          <td [attr.colspan]=\"model.headers.length + (showActions ? 1 : 0)\">\n            <div #footerRef>\n              <ng-content select=\"[data-footer]\"></ng-content>\n            </div>\n            <ng-container *ngIf=\"!footerRef.innerHTML.trim()\">&copy;</ng-container>\n          </td>\n        </tr>\n      </tfoot>\n    </table>\n  </div>\n</div>\n<ng-template #string let-data let-modelpipe=\"pipe\" let-params=\"params\">\n  {{ data | formatData: modelpipe: params }}\n</ng-template>\n<ng-template #boolean let-data>\n  <input type=\"checkbox\" disabled [checked]=\"data\" />\n</ng-template>\n<ng-template #date let-data let-modelpipe=\"pipe\" let-params=\"params\">\n  {{ data | formatData: modelpipe: params }}\n</ng-template>\n<ng-template #enum let-data>\n  {{ data }}\n</ng-template>\n<ng-template #number let-data let-modelpipe=\"pipe\" let-params=\"params\">\n  {{ data | formatData: modelpipe: params }}\n</ng-template>\n",
                    styles: [".colborders col{border-right:1px solid azure}col.last{border-right:none!important}col.first{background-color:#eee}th{background:none}th.sortableHeader{cursor:pointer}.headerText{display:inline-block;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.sort{float:right}.sort button{background-color:transparent!important;border:0;border-radius:0;cursor:pointer;height:16px;padding:1px;width:16px}.sort button.up{left:0}.sort button.down{left:-16px}.rearrange{background-color:transparent;border:none;float:right;height:16px;margin:2px;width:16px}"]
                },] }
    ];
    DataGridComponent.propDecorators = {
        stringFallback: [{ type: core.ViewChild, args: ['string', { static: true },] }],
        string: [{ type: core.ContentChild, args: ['string', { static: true },] }],
        booleanFallback: [{ type: core.ViewChild, args: ['boolean', { static: true },] }],
        boolean: [{ type: core.ContentChild, args: ['boolean', { static: true },] }],
        dateFallback: [{ type: core.ViewChild, args: ['date', { static: true },] }],
        date: [{ type: core.ContentChild, args: ['date', { static: true },] }],
        enumFallback: [{ type: core.ViewChild, args: ['enum', { static: true },] }],
        enum: [{ type: core.ContentChild, args: ['enum', { static: true },] }],
        numberFallback: [{ type: core.ViewChild, args: ['number', { static: true },] }],
        number: [{ type: core.ContentChild, args: ['number', { static: true },] }],
        externals: [{ type: core.Input }],
        columnStyle: [{ type: core.Input }],
        model: [{ type: core.Input }],
        showDeleteButton: [{ type: core.Input }],
        showEditButton: [{ type: core.Input }],
        showActions: [{ type: core.Input }],
        textDeleteButton: [{ type: core.Input }],
        textEditButton: [{ type: core.Input }],
        textButtonsHeader: [{ type: core.Input }],
        textNoItems: [{ type: core.Input }],
        filter: [{ type: core.Input }],
        reArrangeColumns: [{ type: core.Input }],
        editItem: [{ type: core.Output }],
        deleteItem: [{ type: core.Output }]
    };

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    ;
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    /**
     * This pipe allows the dynamic formatting of data. It is just for internal use.
     * The purpose is the forwarding of an externally defined pipe to the pipe formatter within the grid.
     */
    var FormatDataPipe = /** @class */ (function () {
        function FormatDataPipe(injector) {
            this.injector = injector;
        }
        FormatDataPipe.prototype.transform = function (value, pipeToken, pipeArgs) {
            if (!pipeToken) {
                console.log('FormatDataPipe:NoPipeToken');
                return value;
            }
            else {
                console.log('FormatDataPipe:PipeToken:', pipeToken);
                var pipe = this.injector.get(pipeToken);
                return pipe.transform.apply(pipe, __spread([value], pipeArgs || []));
            }
        };
        return FormatDataPipe;
    }());
    FormatDataPipe.decorators = [
        { type: core.Pipe, args: [{
                    name: 'formatData'
                },] }
    ];
    FormatDataPipe.ctorParameters = function () { return [
        { type: core.Injector }
    ]; };

    /**
     * The Editor Widget. Creates a field with all required validators using decorators and forms service.
     */
    var EditorComponent = /** @class */ (function () {
        function EditorComponent() {
            this.params = new Array();
        }
        EditorComponent.prototype.getParams = function (key) {
            return this.first(this.params.filter(function (e) { return e.key === key; }), 0);
        };
        EditorComponent.prototype.first = function (array, n) {
            if (array == null) {
                return void 0;
            }
            if (n == null) {
                return array[0];
            }
            if (n < 0) {
                return [];
            }
            return array.slice(0, n);
        };
        EditorComponent.prototype.ngOnInit = function () {
            var _this = this;
            this.model.formGroup.valueChanges.subscribe(function (data) { return _this.onValueChanged(data); });
            // this is set by FormValidatorService
            var editorModel = this.model.formGroup.__editorModel__;
            // get type from form
            if (editorModel) {
                // get elementary types, this might get overwritten later according to decorators found
                if (typeof editorModel[this.model.name] === 'string') {
                    this.model.type = 'text';
                }
                if (typeof editorModel[this.model.name] === 'boolean') {
                    this.model.type = 'boolean';
                }
                if (typeof editorModel[this.model.name] === 'number') {
                    this.model.type = 'number';
                }
                if (editorModel[this.model.name] instanceof Date) {
                    this.model.type = 'calendar';
                }
                // make an instance to read the properties
                this.model.label = editorModel["__displayName__" + this.model.name] || this.model.label || this.model.name;
                this.model.tooltip = editorModel["__displayDesc__" + this.model.name] || this.model.tooltip || this.model.name;
                // render as range id there is a range definition
                if (editorModel["__hasRangeFrom__" + this.model.name] && Number(editorModel["__hasRangeFrom__" + this.model.name])) {
                    this.model.fromValue = editorModel["__hasRangeFrom__" + this.model.name];
                    this.model.type = 'range';
                }
                if (editorModel["__hasRangeTo__" + this.model.name] && Number(editorModel["__hasRangeTo__" + this.model.name])) {
                    this.model.toValue = editorModel["__hasRangeTo__" + this.model.name];
                    this.model.type = 'range';
                }
                // placeholder
                if (editorModel["__hasWatermark__" + this.model.name]) {
                    this.model.waterMark = editorModel["__watermark__" + this.model.name];
                }
                // templates
                if (editorModel["__hasTemplateHint__" + this.model.name]) {
                    this.model.type = editorModel["__templatehint__" + this.model.name].toLowerCase();
                    if (editorModel["__templatehintParams__" + this.model.name]) {
                        this.params = editorModel["__templatehintParams__" + this.model.name];
                    }
                }
                // render hidden fields as hidden even in forms
                if (editorModel["__isHidden__" + this.model.name]) {
                    this.model.type = 'hidden';
                }
                // check readonly
                if (editorModel["__isReadonly__" + this.model.name]) {
                    this.model.readonly = !!editorModel["__isReadonly__" + this.model.name];
                }
            }
        };
        EditorComponent.prototype.onValueChanged = function (data) {
            // check validation on change
            this.errors = new Array();
            // tslint:disable-next-line:forin
            for (var error in this.model.formGroup.controls[this.model.name].errors) {
                this.errors.push(error);
            }
        };
        return EditorComponent;
    }());
    EditorComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ac-editor',
                    template: "<div class=\"form-group row\"\n    [formGroup]=\"model.formGroup\"\n     *ngIf=\"model.type != 'hidden' && model.formGroup\"\n     [ngClass]=\"{ 'has-danger': !model.formGroup.controls[name].valid && model.formGroup.controls[name].touched }\">\n  <label class=\"col-form-label col-md-3 col-sm-10\"\n    [attr.for]=\"name\"\n    [attr.title]=\"model.tooltip\" >{{ model.label }}{{ model.labelDivider }} </label>\n  <div [ngClass]=\"{ 'col-md-7 col-sm-10': model.inline }\">\n    <textarea *ngIf=\"model.type == 'textarea'\"\n            class=\"form-control\"\n            [id]=\"name\"\n            [readOnly]='model.readonly'\n            [formControlName]=\"name\"\n            [attr.rows]=\"getParams('rows')\"\n            [attr.cols]=\"getParams('cols')\">\n    </textarea>\n    <select *ngIf=\"model.type == 'enum' && model.enumValues\" class=\"form-control\" [id]=\"name\"\n          [formControlName]=\"name\" [disabled]='model.readonly'>\n      <option *ngFor=\"let option of model.enumValues\" [value]=\"option.key\">{{option.val}}</option>\n    </select>\n    <select *ngIf=\"model.type == 'list' && model.listValues\" class=\"form-control\" [id]=\"name\"\n          [formControlName]=\"name\" [disabled]='model.readonly'>\n      <option *ngFor=\"let option of model.listValues\" [value]=\"option.key\">{{option.val}}</option>\n    </select>\n    <input *ngIf=\"model.type == 'range'\" type=\"range\"\n        class=\"form-control\"\n        [placeholder]=\"model.waterMark\"\n        [attr.minvalue]=\"model.fromValue\"\n        [attr.maxvalue]=\"model.toValue\"\n        [attr.min]=\"model.fromValue\"\n        [attr.max]=\"model.toValue\"\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <input *ngIf=\"model.type == 'calendar'\" type=\"date\"\n        class=\"form-control\"\n        [placeholder]=\"model.waterMark\"\n        [readOnly]='model.readonly'\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <input *ngIf=\"model.type == 'number'\" type=\"number\"\n        class=\"form-control\"\n        [placeholder]=\"model.waterMark\"\n        [readOnly]='model.readonly'\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <input *ngIf=\"model.type == 'boolean'\" type=\"checkbox\"\n        class=\"form-control\"\n        [readOnly]='model.readonly'\n        [id]=\"name\"\n        [formControlName]=\"name\"\n        [title]=\"model.tooltip\" />\n    <label *ngIf=\"model.type == 'boolean'\" [attr.for]=\"name\"></label>\n    <input *ngIf=\"model.type == 'text' || model.type == ''\" [placeholder]=\"model.waterMark\"\n        [readOnly]='model.readonly'\n        type=\"text\" class=\"form-control\" [id]=\"name\" [formControlName]=\"name\" />\n    <ng-container *ngIf=\"model.type == 'template'\">\n        <ng-content></ng-content>\n    </ng-container>\n    <span class=\"fa fa-warning text-danger form-control-feedback\"\n        [hidden]=\"model.formGroup.controls[name].valid || model.formGroup.controls[name].pristine\"></span>\n    <small class=\"text-danger\"\n        [hidden]=\"model.formGroup.controls[name].valid || model.formGroup.controls[name].pristine\">\n    <span *ngFor=\"let error of errors\">{{ model.formGroup.controls[name]['messages'][error] }}</span>\n    </small>\n  </div>\n</div>\n<div>\n  <input *ngIf=\"model.type == 'hidden'\" [id]=\"name\" [formControlName]=\"name\" type=\"hidden\" />\n</div>\n",
                    styles: ['input[type="checkbox"] { display: none; }',
                        'input[type="checkbox"] + label:before { font-family: FontAwesome; }',
                        'input[type="checkbox"] + label:before { content: "\\f096"; }',
                        'input[type="checkbox"]:checked + label:before { content: "\\f046"; }', "\n      input[type='checkbox'] + label {\n        display: inline-block;\n        width: 15px;\n        height: 20px;\n        margin: -1px 4px 0 0;\n        vertical-align: middle;\n        cursor: pointer;\n      }\n    "]
                },] }
    ];
    EditorComponent.ctorParameters = function () { return []; };
    EditorComponent.propDecorators = {
        model: [{ type: core.Input }],
        name: [{ type: core.Input }],
        value: [{ type: core.Output }, { type: core.Input }]
    };

    /**
     * This component creates a complete editor from a viewmodel. The viewmodel should have
     * decorators to control the form's behavior. The form can be styled an supports validation.
     * It creates one field for each property not decorated with {@link Hidden}decorator.
     *
     * Example of usage:
     * <example-url>/#/editor</example-url>
     */
    var AutoFormComponent = /** @class */ (function () {
        function AutoFormComponent() {
            /**
             * Ungrouped element will appear at the end, after all groupes.
             * If there are no groups this will be ignored. Optional.
             */
            this.ungroupedAfter = true;
            this.editors = new Array();
            this.groups = new Array();
        }
        AutoFormComponent.prototype.ngOnInit = function () {
            // take all controls as is
            var hasDecorators = this.formGroup.__editorModel__;
            var _loop_1 = function (controlName) {
                if (!this_1.formGroup.controls.hasOwnProperty(controlName)) {
                    return "continue";
                }
                var displayOrder = 0;
                var groupOrder = 0;
                var groupName;
                var groupDesc = void 0;
                var isInGroup = false;
                if (hasDecorators) {
                    displayOrder = this_1.formGroup.__editorModel__["__displayOrder__" + controlName];
                    isInGroup = !!this_1.formGroup.__editorModel__["__isGrouped__" + controlName];
                    if (isInGroup) {
                        groupName = this_1.formGroup.__editorModel__["__groupName__" + controlName];
                        groupOrder = this_1.formGroup.__editorModel__["__groupOrder__" + controlName];
                        groupDesc = this_1.formGroup.__editorModel__["__groupDesc__" + controlName];
                    }
                }
                if (!isInGroup) {
                    // if not part of any group just push to main group
                    this_1.editors.push({
                        key: displayOrder,
                        editor: controlName
                    });
                }
                else {
                    // check if group already exists
                    var existingGroupArray = this_1.groups.filter(function (g) { return g.name === groupName; });
                    var groupExists = existingGroupArray.length === 1;
                    if (!groupExists) {
                        this_1.groups.push({
                            key: groupOrder,
                            name: groupName,
                            desc: groupDesc,
                            editors: new Array()
                        });
                    }
                    // add field to existing group (assume that here the group always exists)
                    var existingGroup = this_1.groups.filter(function (g) { return g.name === groupName; })[0];
                    // and store
                    existingGroup.editors.push({
                        key: displayOrder,
                        editor: controlName
                    });
                }
            };
            var this_1 = this;
            // loop through all fields
            for (var controlName in this.formGroup.controls) {
                _loop_1(controlName);
            }
            // check the display decorator and sort
            this.editors.sort(function (e, n) { return e.key - n.key; });
            // sort groups
            this.groups.sort(function (e, n) { return e.key - n.key; });
        };
        AutoFormComponent.prototype.grouped = function () {
            return this.groups && this.groups.length > 0;
        };
        return AutoFormComponent;
    }());
    AutoFormComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ac-autoform',
                    template: "<ng-content></ng-content>\n<ng-container *ngIf=\"!ungroupedAfter\">\n  <ac-editor *ngFor=\"let editor of editors\" [name]=\"editor.editor\" [formGroup]=\"formGroup\"></ac-editor>\n</ng-container>\n<ng-container *ngIf=\"grouped()\">\n  <fieldset *ngFor=\"let group of groups\">\n    <legend [attr.title]=\"group.desc\" *ngIf=\"group.name\">{{ group.name }}</legend>\n    <ac-editor *ngFor=\"let editor of group.editors\" [name]=\"editor.editor\" [formGroup]=\"formGroup\"></ac-editor>\n  </fieldset>\n</ng-container>\n<ng-container *ngIf=\"ungroupedAfter\">\n  <ac-editor *ngFor=\"let editor of editors\" [name]=\"editor.editor\" [formGroup]=\"formGroup\"></ac-editor>\n</ng-container>\n",
                    styles: ['fieldset { border-top: 1px silver solid; padding: 10px; }',
                        'legend { width: auto; padding-left: 10px; padding-right: 10px; font-size: 1em;}']
                },] }
    ];
    AutoFormComponent.ctorParameters = function () { return []; };
    AutoFormComponent.propDecorators = {
        formGroup: [{ type: core.Input }],
        ungroupedAfter: [{ type: core.Input }]
    };

    /**
     * The treeview can held any tree of nodes, which can be styles, linked, and bound to events.
     * Clicking on the items fires various events.
     *
     * The component creates a treeview that can handle {@link TreeNodeModel objects that create text, checkboxes, or}highlights.
     * A node can have an icon. The icons are based on FontAwesome's css classes.
     *
     * Example of usage:
     * <example-url>/#/widget/tree</example-url>
     */
    var TreeViewComponent = /** @class */ (function () {
        function TreeViewComponent() {
            this.nodeClick = new core.EventEmitter();
            this.checkChanged = new core.EventEmitter();
            this.selectedChanged = new core.EventEmitter();
            this.collapseChanged = new core.EventEmitter();
        }
        TreeViewComponent.prototype.onNodeClick = function (node) {
            this.nodeClick.emit(node);
        };
        TreeViewComponent.prototype.onCheckChanged = function (node) {
            this.checkChanged.emit(node);
        };
        TreeViewComponent.prototype.onSelectedChanged = function (node) {
            this.selectedChanged.emit(node);
        };
        TreeViewComponent.prototype.onCollapseChanged = function (node) {
            this.collapseChanged.emit(node);
        };
        return TreeViewComponent;
    }());
    TreeViewComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ac-tree',
                    template: "<ul class=\"treeview\">\n  <ac-treenode\n    [node]=\"nodes\"\n    (nodeClick)=\"onNodeClick($event)\"\n    (checkChanged)=\"onCheckChanged($event)\"\n    (selectedChanged)=\"onSelectedChanged($event)\"\n    (collapseChanged)=\"onCollapseChanged($event)\"\n  ></ac-treenode>\n</ul>\n",
                    styles: ['.treeview { list-style: none; margin-left: -25px; }']
                },] }
    ];
    TreeViewComponent.ctorParameters = function () { return []; };
    TreeViewComponent.propDecorators = {
        nodes: [{ type: core.Input }],
        nodeClick: [{ type: core.Output }],
        checkChanged: [{ type: core.Output }],
        selectedChanged: [{ type: core.Output }],
        collapseChanged: [{ type: core.Output }]
    };

    /**
     * The TreeNodeModel class represents a single TreeNodeModel for the treeview. Actually, this renders the real UI.
     *
     * The component needs this input value:
     *
     * * @Input() node: AcTextTreeNode;
     *
     * The several options are set through the AcTreeNodeOptions object that is being held by the
     * AcTextTreeNode class. This includes colors, icons, and other styles.
     *
     * The component can fire these events:
     *
     * * @Output() nodeClick: EventEmitter<AcTreeNode>;
     * * @Output() checkChanged: EventEmitter<AcTreeNode>;
     * * @Output() selectedChanged: EventEmitter<AcTreeNode>;
     * * @Output() collapseChanged: EventEmitter<AcTreeNode>;
     *
     */
    var TreeViewNodeComponent = /** @class */ (function () {
        function TreeViewNodeComponent(el, renderer) {
            this.el = el;
            this.renderer = renderer;
            this.isExpanded = false;
            this.nodeClick = new core.EventEmitter();
            this.checkChanged = new core.EventEmitter();
            this.selectedChanged = new core.EventEmitter();
            this.collapseChanged = new core.EventEmitter();
        }
        TreeViewNodeComponent.prototype.ngOnInit = function () {
            var _this = this;
            // set HTML according to options
            this.collapseClasses = new Array();
            this.iconClasses = new Array();
            // expander area with icon
            if (this.node) {
                // expect a font-awesome class with or without fa class
                if (this.node.options && this.node.options.icon) {
                    if (this.node.options.icon.indexOf(TreeViewNodeComponent.pfxIcon + " ") !== 0) {
                        this.iconClasses.push('fa');
                    }
                    this.iconClasses.push(this.node.options.icon);
                }
                if (this.node.options && this.node.options.href) {
                    this.href = this.node.options.href;
                }
                else {
                    this.href = '';
                }
                // open/close area for elements with children
                if (this.node.hasChildren) {
                    this.collapseClasses.push('ac-icon'); // base class
                    this.collapseClasses.push(TreeViewNodeComponent.pfxIcon);
                    this.collapseClasses.push(TreeViewNodeComponent.opnIcon);
                    // collapsed by default
                    this.node.state &= ~core$1.TreeNodeState.checked;
                }
                if (this.node.options && this.node.options.color) {
                    this.foreColor = this.node.options.color;
                }
                if (this.node.options && this.node.options.backColor) {
                    this.backColor = this.node.options.backColor;
                }
                this.node.stateChange.subscribe(function (se) {
                    _this.collapseChanged.emit(_this.node);
                    if (!_this.node.stateIsExpandend) {
                        _this.collapseClasses = _this.collapseClasses.filter(function (s) { return s !== TreeViewNodeComponent.clsIcon; });
                        _this.collapseClasses.push(TreeViewNodeComponent.opnIcon);
                    }
                    else {
                        _this.collapseClasses = _this.collapseClasses.filter(function (s) { return s !== TreeViewNodeComponent.opnIcon; });
                        _this.collapseClasses.push(TreeViewNodeComponent.clsIcon);
                    }
                    _this.isExpanded = _this.node.stateIsExpandend;
                });
            }
        };
        // forward events in the node tree
        TreeViewNodeComponent.prototype.onNodeClick = function (node) {
            this.nodeClick.emit(node);
        };
        TreeViewNodeComponent.prototype.onCheckChanged = function (node) {
            this.checkChanged.emit(node);
        };
        TreeViewNodeComponent.prototype.onSelectedChanged = function (node) {
            this.selectedChanged.emit(node);
        };
        TreeViewNodeComponent.prototype.onCollapseChanged = function (node) {
            if (this.node.options.collapsable && this.node.hasChildren) {
                this.collapseChanged.emit(node);
            }
        };
        TreeViewNodeComponent.prototype.handleCheckChange = function () {
            if (this.node.options.checkable && !this.node.stateIsDisabled) {
                if (this.node.state & core$1.TreeNodeState.checked) {
                    this.node.state &= ~core$1.TreeNodeState.checked;
                }
                else {
                    this.node.state |= core$1.TreeNodeState.checked;
                }
                this.checkChanged.emit(this.node);
            }
        };
        TreeViewNodeComponent.prototype.handleClick = function ($event) {
            $event.stopPropagation();
            // always emit click
            this.nodeClick.emit(this.node);
        };
        TreeViewNodeComponent.prototype.handleCollapse = function () {
            // if collapsable handle icons and view state
            if (this.node.options.collapsable) {
                // toggle state
                if (this.node.state && this.node.state & core$1.TreeNodeState.expanded) {
                    this.node.state &= ~core$1.TreeNodeState.expanded;
                }
                else {
                    this.node.state |= core$1.TreeNodeState.expanded;
                }
            }
        };
        TreeViewNodeComponent.prototype.handlePreSelection = function (state) {
            // handle the states according to options
            if (this.node.options.selectable && !this.node.stateIsDisabled) {
                if (state) {
                    if (this.node.options && this.node.options.color) {
                        this.foreColor = this.node.options.backColor;
                    }
                    else {
                        this.foreColor = '#fff';
                    }
                    if (this.node.options && this.node.options.backColor) {
                        this.backColor = this.node.options.color;
                    }
                    else {
                        this.backColor = '#000';
                    }
                }
                else {
                    if (this.node.options && this.node.options.color) {
                        this.foreColor = this.node.options.color;
                    }
                    else {
                        this.foreColor = '#000';
                    }
                    if (this.node.options && this.node.options.backColor) {
                        this.backColor = this.node.options.backColor;
                    }
                    else {
                        this.backColor = '#fff';
                    }
                }
                this.preSelectState = state;
            }
        };
        TreeViewNodeComponent.prototype.handleSelection = function ($event) {
            // handle the states according to options
            if (this.node.options.selectable &&
                !this.node.stateIsDisabled &&
                this.preSelectState === true) {
                if (this.node.state & core$1.TreeNodeState.selected) {
                    this.node.state &= ~core$1.TreeNodeState.selected;
                }
                else {
                    this.node.state |= core$1.TreeNodeState.selected;
                }
                if (this.node.stateIsSelected) {
                    if (this.node.options && this.node.options.color) {
                        this.foreColor = this.node.options.backColor;
                    }
                    else {
                        this.foreColor = '#fff';
                    }
                    if (this.node.options && this.node.options.backColor) {
                        this.backColor = this.node.options.color;
                    }
                    else {
                        this.backColor = '#000';
                    }
                }
                else {
                    if (this.node.options && this.node.options.color) {
                        this.foreColor = this.node.options.color;
                    }
                    else {
                        this.foreColor = '#000';
                    }
                    if (this.node.options && this.node.options.backColor) {
                        this.backColor = this.node.options.backColor;
                    }
                    else {
                        this.backColor = '#fff';
                    }
                }
                this.selectedChanged.emit(this.node);
            }
            if (!this.href) {
                $event.preventDefault();
                $event.stopPropagation();
            }
        };
        return TreeViewNodeComponent;
    }());
    TreeViewNodeComponent.pfxIcon = 'fa';
    TreeViewNodeComponent.opnIcon = 'fa-plus';
    TreeViewNodeComponent.clsIcon = 'fa-minus';
    TreeViewNodeComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'ac-treenode',
                    template: "<li class=\"treeview\" (click)=\"handleClick($event)\">\n    <i class=\"ac-collapse\" [ngClass]=\"collapseClasses\" *ngIf=\"node.hasChildren\" (click)=\"handleCollapse()\"></i>\n    <i class=\"ac-collapse\" *ngIf=\"!node.hasChildren\"></i>\n    <i class=\"ac-icon\" [ngClass]=\"iconClasses\" [style.color]=\"node.options.iconColor\" *ngIf=\"!node.options.checkable\"></i>\n    <input type=\"checkbox\" [id]=\"node.name\" *ngIf=\"node.options.checkable\" \n           [checked]=\"node.stateIsChecked\" (click)=\"handleCheckChange()\">\n    <label [attr.for]=\"node.name\"></label>\n    <a class=\"ac-container\"\n          [href]=\"href\"\n          [style.color]=\"foreColor\" \n          [style.background-color]=\"backColor\" \n          (mouseover)=\"handlePreSelection(true)\"\n          (mouseout)=\"handlePreSelection(false)\"\n          (click)=\"handleSelection($event)\">\n        {{ node.text }}\n    </a>\n    <ul class=\"treeview\" *ngIf=\"node.hasChildren\" [hidden]=\"!isExpanded\">\n        <ac-treenode *ngFor=\"let child of node.children\" \n                     [node]=\"child\"\n                     (nodeClick)=\"onNodeClick($event)\" \n                     (checkChanged)=\"onCheckChanged($event)\"\n                     (selectedChanged)=\"onSelectedChanged($event)\"\n                     (collapseChanged)=\"onCollapseChanged($event)\">\n        </ac-treenode>\n    </ul>                   \n</li>",
                    styles: ["ul.treeview{list-style:none;margin-left:-2em}li.treeview{box-sizing:border-box;margin-bottom:3px;margin-left:10px}li.treeview a.ac-container{border-radius:2px;display:inline-block;padding:3px;text-decoration:none}li.treeview input[type=checkbox]{display:none}li.treeview input[type=checkbox]+label:before{content:\"\\f096\";font-family:FontAwesome}li.treeview input[type=checkbox]:checked+label:before{content:\"\\f046\"}li.treeview input[type=checkbox]+label{cursor:pointer;display:inline-block;height:20px;margin:-1px 4px 0 0;vertical-align:middle;width:15px}li.treeview i.ac-collapse{margin-left:-1.7em}li.treeview i.ac-collapse,li.treeview i.ac-icon{cursor:pointer;display:inline-block;width:15px}li.treeview .ac-node-disabled{color:silver;cursor:not-allowed}"]
                },] }
    ];
    TreeViewNodeComponent.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.Renderer2 }
    ]; };
    TreeViewNodeComponent.propDecorators = {
        node: [{ type: core.Input }],
        nodeClick: [{ type: core.Output }],
        checkChanged: [{ type: core.Output }],
        selectedChanged: [{ type: core.Output }],
        collapseChanged: [{ type: core.Output }]
    };

    // import { FormValidatorFromJsonService } from './services/formvalidator-fromjson.service';
    var SVOGV_COMPONENTS = [
        DataGridComponent,
        DataGridPaginationComponent,
        TreeViewComponent,
        TreeViewNodeComponent,
        EditorComponent,
        AutoFormComponent
    ];
    var provider = [core$1.SvogvCoreModule];
    /**
     * The actual SVOGV Module definition.
     */
    var SvogvBootstrapModule = /** @class */ (function () {
        function SvogvBootstrapModule() {
        }
        SvogvBootstrapModule.forRoot = function () {
            return {
                ngModule: SvogvBootstrapModule,
                providers: provider
            };
        };
        return SvogvBootstrapModule;
    }());
    SvogvBootstrapModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule, forms.ReactiveFormsModule, forms.FormsModule],
                    declarations: __spread(SVOGV_COMPONENTS, [FormatDataPipe]),
                    providers: provider,
                    exports: SVOGV_COMPONENTS
                },] }
    ];

    /*
     * Public API Surface of bootstrap-ui
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.AutoFormComponent = AutoFormComponent;
    exports.DataGridComponent = DataGridComponent;
    exports.DataGridPaginationComponent = DataGridPaginationComponent;
    exports.EditorComponent = EditorComponent;
    exports.FormatDataPipe = FormatDataPipe;
    exports.SvogvBootstrapModule = SvogvBootstrapModule;
    exports.TreeViewComponent = TreeViewComponent;
    exports.TreeViewNodeComponent = TreeViewNodeComponent;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=svogv-bootstrap.umd.js.map
