import { OnInit, OnChanges, SimpleChange, EventEmitter } from '@angular/core';
/**
 * The pagination component creates a few buttons to navigate a grid. The underlaying model
 * is going to handle the date on the client. The pagination does not support a server backend,
 * all relevant data must be loaded first.
 *
 * Example of usage:
 * @example
 * ```html
 * <ac-pagination></ac-pagination>
 * ```
 *
 * <example-url>/#/widget/grid</example-url>
 */
export declare class DataGridPaginationComponent implements OnInit, OnChanges {
    /**
     * The parent component calculated the pages the component can handle.
     */
    maxPageIndex: number;
    /**
     * An event fired once the user has changed the page by clicking a button.
     */
    pageNumberChanged: EventEmitter<any>;
    /**
     * The size for pagination, according bootstrap rules. ('sm' == 'pagination-sm', 'lg' = 'pagination-lg').
     * Default is medium.
     */
    size: string;
    currentPageNumber: number;
    ngOnInit(): void;
    ngOnChanges(changes: {
        [propertyName: string]: SimpleChange;
    }): void;
    setCurrentPage(pageNumber: number, event?: MouseEvent): void;
    range(min: number, max: number): number[];
    get pageStartNumber(): number;
    get pageEndNumber(): number;
}
//# sourceMappingURL=datagridpagination.component.d.ts.map